#!/usr/bin/python3
# -*-coding:Utf-8 -*

import sys
import matplotlib
# Make sure that we are using QT5
matplotlib.use('Qt5Agg')
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
import numpy as np
import scipy.signal
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
import matplotlib.pyplot as plt


class Window(QWidget):
    def __init__(self, parent=None):
        super(Window, self).__init__(parent)
        self.fct="Sinusoïde"

        self.setWindowTitle("Redresseur PD2 tout thyristor")

        
        # a figure instance to plot on
        self.figure1 = plt.figure()
        self.figure1.patch.set_facecolor('white')
        # create an axis
        self.ax = self.figure1.add_subplot(211)

        # create an other axis
        self.ax2 = self.figure1.add_subplot(212)
        
        # this is the Canvas Widget that displays the `figure`
        # it takes the `figure` instance as a parameter to __init__
        self.canvas1 = FigureCanvas(self.figure1)

        # this is the Navigation widget
        # it takes the Canvas widget and a parent
        self.toolbar1 = NavigationToolbar(self.canvas1, self)
        
        #  slider
        self.labelSlider=QLabel()
        self.labelSlider.setText("Retard\n à \n l'amorçage")
        
        self.slider=QSlider()
        self.MaxSlid=180
        self.slider.setOrientation(Qt.Vertical)
        self.slider.setTickPosition(QSlider.TicksAbove)
        self.slider.setMaximum(self.MaxSlid)
        self.slider.setSingleStep(1)
        self.slider.setPageStep(10)
        self.slider.setTickInterval(10)     
        self.slider.setValue(0)
        self.slider.valueChanged.connect(self.plotSlider)
        self.slider.setToolTip('retard : souris\n ou clavier')#infobulle
        
        self.editSlider=QLineEdit()
        self.editSlider.setText("0")
        self.editSlider.setMaximumWidth(50)
        self.editSlider.textChanged.connect(self.plotLine)
        
        #  commentaires
        self.labelC=QLabel()
        self.labelC.setText("Le curseur règle le retard à l'amorçage")
        self.labelC.setGeometry(0,0,180,30)
        #  déco
        self.labelC.setStyleSheet("""
        .QLabel {
            border: 2px solid maroon;
            border-radius: 10px;
            background-color: 'white';
            text-align : center;
            }
        """)  #  text-align n'a aucun effet. d'où la ligne suivante
        self.labelC.setAlignment(Qt.AlignHCenter)

        # set the layout______________________________________________________________________________________
        layout = QVBoxLayout()  #  layout principal
        layoutFig = QHBoxLayout()  #  contient le graphe et le slider
        layoutSlid = QVBoxLayout()  #  contient le slider, son label et la zone d'édition
        layoutGraph = QVBoxLayout()  #  contient le graphe et ses menus
        
        layoutSlid.addWidget(self.labelSlider)
        layoutSlid.addWidget(self.slider)
        layoutSlid.addWidget(self.editSlider)
        
        layoutGraph.addWidget(self.canvas1)
        layoutGraph.addWidget(self.toolbar1)
        
        #  layoutFig.addWidget(self.stackFig)
        layoutFig.addLayout(layoutGraph)#insère layoutGraph  dans layoutFig
        layoutFig.addLayout(layoutSlid)#insère layoutSlid  dans layoutFig
               
        layout.addLayout(layoutFig)#insère layoutFig dans le layout principal
        layout.addWidget(self.labelC)
        
        self.setLayout(layout)
        #____________________________________________________________________________________________________
        
        self.n = 10000  #  nombre de points d'échantillonage

        #  fond de courbes
        self.Fe = 10000.0
        self.Te = 1/self.Fe
        self.Tt = np.arange(self.n)*self.Te
        self.v1 = np.sin(2*np.pi*50.0*self.Tt)
        self.v2 = -np.sin(2*np.pi*50.0*self.Tt)
        self.tension = []
        self.courant = []
        #  courbe nulle pour marquer l'self.axe
        self.nulle = scipy.linspace(0.0,0.0,num=400)
        self.flagEditS = False
        self.plot()

    def fonction(self,f,fe):
        T = 1/f
        for t in self.Tt:
            
            reste = f*(t%T)  #  temps ramené sur une période normalisé de 0à1
            t1 = self.slider.value()*5.55556e-5  #  retard à l'amorçage en s
            
            if reste <= 0.5:
                if reste < t1/T:
                    self.tension.append(-np.sin(2*np.pi*f*t))
                    self.courant.append(-1.0)
                else:
                    self.tension.append(np.sin(2*np.pi*f*t))
                    self.courant.append(1.0)
            else:
                if reste < (t1/T+0.5):
                    self.tension.append(np.sin(2*np.pi*f*t))
                    self.courant.append(1.0)
                else:
                    self.tension.append(-np.sin(2*np.pi*f*t)) 
                    self.courant.append(-1.0)

        
    def plotSlider(self):
        if not self.flagEditS: self.editSlider.setText(str(self.slider.value()))
        self.plot()

    def plotLine(self):
        self.flagEditS = True
        self.slider.setValue(float(self.editSlider.text()))
        self.flagEditS = False

    def plot(self):
        self.tension = []
        self.courant = []
        f = 50.0
        nPoints = 400
        self.fonction(f,self.Fe)

        # discards the old graph
        self.ax.clear()
        self.ax2.clear()

        # plot data
        self.ax2.plot(self.Tt[:nPoints], self.courant[:nPoints], linewidth=2)
        self.ax.plot(self.Tt[:nPoints], self.tension[:nPoints], linewidth=2)
        
        self.ax2.plot(self.Tt[:nPoints], self.v1[:nPoints], linewidth=2)
        self.ax.plot(self.Tt[:nPoints], self.v1[:nPoints], 'grey', linewidth=1)
        self.ax.plot(self.Tt[:nPoints], self.v2[:nPoints], 'grey', linewidth=1)
        self.ax.plot(self.Tt[:nPoints], self.nulle[:nPoints], 'black')
        self.ax2.plot(self.Tt[:nPoints], self.nulle[:nPoints], 'black')
        self.ax2.set_xlabel("temps")
        self.ax2.set_ylabel("Courant réseau i1 et v1")
        self.ax.set_ylabel("Tension redressée")
        self.ax.set_ylim([-1.1, 1.1])
        self.ax2.set_ylim([-1.1, 1.1])

        self.ax.grid(True)
        self.ax2.grid(True)

        # refresh canvas
        self.canvas1.draw()


if __name__ == '__main__':
    app = QApplication(sys.argv)

    main = Window()
    main.show()
    app.exec_()
